//=============================================================================
//  MuseScore
//
//  Color Cello Strings (for half and 1st position including extensions) plugin
//  for MuseScore 2.0 
//  Copyright (C)2017 Amandine Fressier
//  Inspired by colorstrings by underquark and colorhalfsteps by Mike Magatagan
//  modified for MuseScore 2.0 by Chad Kurszewski
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================
import QtQuick 2.0
import MuseScore 1.0

MuseScore {
      version:  "2.0"
      description: "This plugin colors notes function of the strings for Cello.  Run it a second time to change back to all black notes."
      menuPath: "Plugins.Notes.Color Cello Strings"

      property variant black : "#000000";
	  	 

      // Apply the given function to all notes in selection
      // or, if nothing is selected, in the entire score

      function applyToNotesInSelection(func) {
            var cursor = curScore.newCursor();
            cursor.rewind(1);
            var startStaff;
            var endStaff;
            var endTick;
            var fullScore = false;
            if (!cursor.segment) { // no selection
                  fullScore = true;
                  startStaff = 0; // start with 1st staff
                  endStaff = curScore.nstaves - 1; // and end with last
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2);
                  if (cursor.tick == 0) {
                        // this happens when the selection includes
                        // the last measure of the score.
                        // rewind(2) goes behind the last segment (where
                        // there's none) and sets tick=0
                        endTick = curScore.lastSegment.tick + 1;
                  } else {
                        endTick = cursor.tick;
                  }
                  endStaff = cursor.staffIdx;
            }
            console.log(startStaff + " - " + endStaff + " - " + endTick)
            for (var staff = startStaff; staff <= endStaff; staff++) {
                  for (var voice = 0; voice < 4; voice++) {
                        cursor.rewind(1); // sets voice to 0
                        cursor.voice = voice; //voice has to be set after goTo
                        cursor.staffIdx = staff;

                        if (fullScore)
                              cursor.rewind(0) // if no selection, beginning of score

                        while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                              if (cursor.element && cursor.element.type == Element.CHORD) {
                                    var graceChords = cursor.element.graceNotes;
                                    for (var i = 0; i < graceChords.length; i++) {
                                          // iterate through all grace chords
                                          var notes = graceChords[i].notes;
                                          func(note);
                                    }
                                    var notes = cursor.element.notes;
                                    for (var i = 0; i < notes.length; i++) {
                                          var note = notes[i];
                                          func(note);
                                    }
                              }
                              cursor.next();
                        }
                  }
            }
      }

            function colorNote(note) {
             if (note.color == black)
				   switch (note.pitch)
                       {                         case 36:  // C - C-String open		                      
						 case 37:  //					                      
						 case 38:  //					                      
						 case 39:  //					                      
						 case 40:  //					                      
						 case 41:  //					                      
						 case 42: //					                                          
                  	     note.color = 'purple';					         
					           break;
						 case 43:  // G - G-String open
						 case 44:  // G# - G-String 1st finger
						 case 45:  // A
						 case 46:  // A#
						 case 47:  // B						 						
					         case 48:  // C
					         case 49:  // C#
                  	     note.color = 'blue';					         
					           break;
					         case 50:  // D - D-String open
					         case 51:  // D# - D-String 1st finger
					         case 52:  // E
					         case 53:  // F
					         case 54:  // F#
					         case 55:  // G
					         case 56:  // G#
                  	     note.color = 'gold';					         
					           break;					         
					         case 57:  // A - A-String open
					         case 58:  // A# - A-String 1st finger
					         case 59:  //
					         case 60:  //
					         case 61:  //
					         case 62:  //
					         case 63:  // D#
                  	     note.color = 'green';					         
					           break;					         
					         					                      
	                     default:  // Beyond the ranges here either MuseScore default colors apply or the note is Black
                       	  note.color = black;
                          break;
                       }                  
           else
                  note.color = black;
      }

      onRun: {
            console.log("hello Color Cello Strings");

            if (typeof curScore === 'undefined')
                  Qt.quit();

            applyToNotesInSelection(colorNote)

            Qt.quit();
         }
}